=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2009-02-25 15:36:48 +0000
+++ bzrlib/branch.py	2009-02-25 23:52:42 +0000
@@ -45,6 +45,7 @@
 
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.hooks import Hooks
+from bzrlib.inter import InterObject
 from bzrlib import registry
 from bzrlib.symbol_versioning import (
     deprecated_in,
@@ -704,44 +705,8 @@
             information. This can be None.
         :return: None
         """
-        other.lock_read()
-        try:
-            other_revno, other_last_revision = other.last_revision_info()
-            stop_revno = None # unknown
-            if stop_revision is None:
-                stop_revision = other_last_revision
-                if _mod_revision.is_null(stop_revision):
-                    # if there are no commits, we're done.
-                    return
-                stop_revno = other_revno
-
-            # what's the current last revision, before we fetch [and change it
-            # possibly]
-            last_rev = _mod_revision.ensure_null(self.last_revision())
-            # we fetch here so that we don't process data twice in the common
-            # case of having something to pull, and so that the check for
-            # already merged can operate on the just fetched graph, which will
-            # be cached in memory.
-            self.fetch(other, stop_revision)
-            # Check to see if one is an ancestor of the other
-            if not overwrite:
-                if graph is None:
-                    graph = self.repository.get_graph()
-                if self._check_if_descendant_or_diverged(
-                        stop_revision, last_rev, graph, other):
-                    # stop_revision is a descendant of last_rev, but we aren't
-                    # overwriting, so we're done.
-                    return
-            if stop_revno is None:
-                if graph is None:
-                    graph = self.repository.get_graph()
-                this_revno, this_last_revision = self.last_revision_info()
-                stop_revno = graph.find_distance_to_null(stop_revision,
-                                [(other_last_revision, other_revno),
-                                 (this_last_revision, this_revno)])
-            self.set_last_revision_info(stop_revno, stop_revision)
-        finally:
-            other.unlock()
+        return InterBranch.get(other, self).update_revisions(stop_revision,
+            overwrite, graph)
 
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
@@ -2744,3 +2709,93 @@
     else:
         target.unlock()
         return result
+
+
+class InterBranch(InterObject):
+    """This class represents operations taking place between two branches.
+
+    Its instances have methods like pull() and push() and contain
+    references to the source and target repositories these operations
+    can be carried out on.
+    """
+
+    _optimisers = []
+    """The available optimised InterBranch types."""
+
+    @staticmethod
+    def _get_branch_formats_to_test():
+        """Return a tuple with the Branch formats to use when testing."""
+        raise NotImplementedError(self._get_branch_formats_to_test)
+
+    def update_revisions(self, stop_revision=None, overwrite=False,
+                         graph=None):
+        """Pull in new perfect-fit revisions.
+
+        :param stop_revision: Updated until the given revision
+        :param overwrite: Always set the branch pointer, rather than checking
+            to see if it is a proper descendant.
+        :param graph: A Graph object that can be used to query history
+            information. This can be None.
+        :return: None
+        """
+        raise NotImplementedError(self.update_revisions)
+
+
+class GenericInterBranch(InterBranch):
+    """InterBranch implementation that uses public Branch functions.
+    """
+
+    @staticmethod
+    def _get_branch_formats_to_test():
+        return BranchFormat._default_format, BranchFormat._default_format
+
+    def update_revisions(self, stop_revision=None, overwrite=False,
+        graph=None):
+        """See InterBranch.update_revisions()."""
+        self.source.lock_read()
+        try:
+            other_revno, other_last_revision = self.source.last_revision_info()
+            stop_revno = None # unknown
+            if stop_revision is None:
+                stop_revision = other_last_revision
+                if _mod_revision.is_null(stop_revision):
+                    # if there are no commits, we're done.
+                    return
+                stop_revno = other_revno
+
+            # what's the current last revision, before we fetch [and change it
+            # possibly]
+            last_rev = _mod_revision.ensure_null(self.target.last_revision())
+            # we fetch here so that we don't process data twice in the common
+            # case of having something to pull, and so that the check for
+            # already merged can operate on the just fetched graph, which will
+            # be cached in memory.
+            self.target.fetch(self.source, stop_revision)
+            # Check to see if one is an ancestor of the other
+            if not overwrite:
+                if graph is None:
+                    graph = self.target.repository.get_graph()
+                if self.target._check_if_descendant_or_diverged(
+                        stop_revision, last_rev, graph, self.source):
+                    # stop_revision is a descendant of last_rev, but we aren't
+                    # overwriting, so we're done.
+                    return
+            if stop_revno is None:
+                if graph is None:
+                    graph = self.target.repository.get_graph()
+                this_revno, this_last_revision = \
+                        self.target.last_revision_info()
+                stop_revno = graph.find_distance_to_null(stop_revision,
+                                [(other_last_revision, other_revno),
+                                 (this_last_revision, this_revno)])
+            self.target.set_last_revision_info(stop_revno, stop_revision)
+        finally:
+            self.source.unlock()
+
+    @classmethod
+    def is_compatible(self, source, target):
+        # GenericBranch uses the public API, so always compatible
+        return True
+
+
+InterBranch.register_optimiser(GenericInterBranch)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-02-25 14:36:59 +0000
+++ bzrlib/tests/__init__.py	2009-02-25 23:52:42 +0000
@@ -2798,6 +2798,7 @@
                    'bzrlib.tests.interrepository_implementations',
                    'bzrlib.tests.intertree_implementations',
                    'bzrlib.tests.inventory_implementations',
+                   'bzrlib.tests.per_interbranch',
                    'bzrlib.tests.per_lock',
                    'bzrlib.tests.per_repository',
                    'bzrlib.tests.per_repository_reference',

=== added directory 'bzrlib/tests/per_interbranch'
=== added file 'bzrlib/tests/per_interbranch/__init__.py'
--- bzrlib/tests/per_interbranch/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/__init__.py	2009-02-25 22:43:23 +0000
@@ -0,0 +1,145 @@
+# Copyright (C) 2009 Canonical Ltd
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""InterBranch implementation tests for bzr.
+
+These test the conformance of all the interbranch variations to the
+expected API including generally applicable corner cases.
+Specific tests for individual formats are in the tests for the formats
+itself rather than in tests/per_interbranch/*.py.
+"""
+
+
+from bzrlib.branch import (
+                           GenericInterBranch,
+                           InterBranch,
+                           )
+from bzrlib.errors import (
+    FileExists,
+    UninitializableFormat,
+    )
+from bzrlib.tests import (
+                          adapt_modules,
+                          TestScenarioApplier,
+                          )
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.transport import get_transport
+
+
+class InterBranchTestProviderAdapter(TestScenarioApplier):
+    """A tool to generate a suite testing multiple inter branch formats.
+
+    This is done by copying the test once for each interbranch provider and
+    injecting the branch_format_from and branch_to_format classes into each
+    copy.  Each copy is also given a new id() to make it easy to identify.
+    """
+
+    def __init__(self, formats):
+        TestScenarioApplier.__init__(self)
+        self.scenarios = self.formats_to_scenarios(formats)
+
+    def formats_to_scenarios(self, formats):
+        """Transform the input formats to a list of scenarios.
+
+        :param formats: A list of tuples:
+            (interbranch_class, branch_format_from, branch_format_to).
+        """
+        result = []
+        for interbranch_class, branch_format_from, branch_format_to in formats:
+            id = '%s,%s,%s' % (interbranch_class.__name__,
+                                branch_format_from.__class__.__name__,
+                                branch_format_to.__class__.__name__)
+            scenario = (id,
+                {
+                 "branch_format_from":branch_format_from,
+                 "interbranch_class":interbranch_class,
+                 "branch_format_to":branch_format_to,
+                 })
+            result.append(scenario)
+        return result
+
+    @staticmethod
+    def default_test_list():
+        """Generate the default list of interbranch permutations to test."""
+        result = []
+        # test the default InterBranch between format 6 and the current
+        # default format.
+        for optimiser_class in InterBranch._optimisers:
+            format_from_test, format_to_test = \
+                optimiser_class._get_branch_formats_to_test()
+            if format_to_test is not None:
+                result.append((optimiser_class,
+                               format_from_test, format_to_test))
+        # if there are specific combinations we want to use, we can add them
+        # here.
+        return result
+
+
+class TestCaseWithInterBranch(TestCaseWithBzrDir):
+
+    def setUp(self):
+        super(TestCaseWithInterBranch, self).setUp()
+
+    def make_branch(self, relpath, format=None):
+        repo = self.make_repository(relpath, format=format)
+        return repo.bzrdir.create_branch()
+
+    def make_bzrdir(self, relpath, format=None):
+        try:
+            url = self.get_url(relpath)
+            segments = url.split('/')
+            if segments and segments[-1] not in ('', '.'):
+                parent = '/'.join(segments[:-1])
+                t = get_transport(parent)
+                try:
+                    t.mkdir(segments[-1])
+                except FileExists:
+                    pass
+            if format is None:
+                format = self.branch_format_from._matchingbzrdir
+            return format.initialize(url)
+        except UninitializableFormat:
+            raise TestSkipped("Format %s is not initializable." % format)
+
+    def make_repository(self, relpath, format=None):
+        made_control = self.make_bzrdir(relpath, format=format)
+        return made_control.create_repository()
+
+    def make_to_bzrdir(self, relpath):
+        return self.make_bzrdir(relpath,
+            self.branch_format_to._matchingbzrdir)
+
+    def make_to_branch(self, relpath):
+        made_control = self.make_to_bzrdir(relpath)
+        return self.branch_format_to.initialize(made_control)
+
+
+def load_tests(basic_tests, module, loader):
+    result = loader.suiteClass()
+    # add the tests for this module
+    result.addTests(basic_tests)
+
+    test_interbranch_implementations = [
+        'bzrlib.tests.per_interbranch.test_update_revisions',
+        ]
+    adapter = InterBranchTestProviderAdapter(
+        InterBranchTestProviderAdapter.default_test_list()
+        )
+    # add the tests for the sub modules
+    adapt_modules(test_interbranch_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/per_interbranch/test_update_revisions.py'
--- bzrlib/tests/per_interbranch/test_update_revisions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_update_revisions.py	2009-02-25 02:35:39 +0000
@@ -0,0 +1,70 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib import (
+    errors,
+    )
+from bzrlib.tests.per_interbranch import (
+    TestCaseWithInterBranch,
+    )
+
+
+class TestUpdateRevisions(TestCaseWithInterBranch):
+
+    def setUp(self):
+        super(TestUpdateRevisions, self).setUp()
+        self.tree1 = self.make_branch_and_tree('tree1')
+        rev1 = self.tree1.commit('one')
+        dir2 = self.make_to_bzrdir('tree2')
+        dir2.create_repository().fetch(self.tree1.branch.repository)
+        branch2 = self.tree1.branch.sprout(dir2)
+        self.tree2 = dir2.create_workingtree()
+
+    def test_accepts_graph(self):
+        # An implementation may not use it, but it should allow a 'graph' to be
+        # supplied
+        rev2 = self.tree2.commit('two')
+
+        self.tree1.lock_write()
+        self.addCleanup(self.tree1.unlock)
+        self.tree2.lock_read()
+        self.addCleanup(self.tree2.unlock)
+        graph = self.tree2.branch.repository.get_graph(
+            self.tree1.branch.repository)
+
+        self.tree1.branch.update_revisions(self.tree2.branch, graph=graph)
+        self.assertEqual((2, rev2), self.tree1.branch.last_revision_info())
+
+    def test_overwrite_ignores_diverged(self):
+        rev2 = self.tree1.commit('two')
+        rev2b = self.tree2.commit('alt two')
+
+        self.assertRaises(errors.DivergedBranches,
+                          self.tree1.branch.update_revisions,
+                          self.tree2.branch, overwrite=False)
+        # However, the revision should be copied into the repository
+        self.assertTrue(self.tree1.branch.repository.has_revision(rev2b))
+
+        self.tree1.branch.update_revisions(self.tree2.branch, overwrite=True)
+        self.assertEqual((2, rev2b), self.tree1.branch.last_revision_info())
+
+    def test_ignores_older_unless_overwrite(self):
+        rev2 = self.tree1.commit('two')
+
+        self.tree1.branch.update_revisions(self.tree2.branch)
+        self.assertEqual((2, rev2), self.tree1.branch.last_revision_info())
+
+        self.tree1.branch.update_revisions(self.tree2.branch, overwrite=True)

